/**
 * @copyright nhcarrigan
 * @license Naomi's Public License
 * @author Naomi Carrigan
 */
import formParser from "@fastify/formbody";
import fastify from "fastify";
import { formHtml, revokeHtml } from "../config/form.js";
import { generateHtml } from "../modules/generateHtml.js";
import { updateCache } from "../modules/updateCache.js";
import type { App } from "../interfaces/app.js";

/**
 * Instantiates the fastify server.
 * @param app - The application instance.
 */
// eslint-disable-next-line max-lines-per-function -- May refactor?
export const serve = async(app: App): Promise<void> => {
  const server = fastify({
    logger: false,
  });

  server.register(formParser);

  server.get("/", (_request, response) => {
    response.header("Content-Type", "text/html");
    response.send(generateHtml(app));
  });

  server.get("/log", (_request, response) => {
    response.header("Content-Type", "text/html");
    response.send(formHtml);
  });

  server.get("/revoke", (_request, response) => {
    response.header("Content-Type", "text/html");
    response.send(revokeHtml);
  });

  server.post<{
    // eslint-disable-next-line @typescript-eslint/naming-convention -- Required convention for Fastify.
    Body: {
      uuid:     string;
      platform: string;
      action:   string;
      reason:   string;
      token:    string;
    };
  }>("/log", async(request, response) => {
    try {
      const { body } = request;

      const token = await app.database.tokens.findUnique({
        where: {
          token: body.token,
        },
      });

      if (!token) {
        response.status(401);
        response.send("Invalid token.");
        return;
      }

      const number = await app.database.sanctions.count();
      await app.database.sanctions.create({
        data: {
          action:    body.action,
          moderator: token.username,
          number:    number + 1,
          platform:  body.platform,
          reason:    body.reason,
          uuid:      body.uuid,
        },
      });

      response.status(200);
      response.send(
        `Logged #${String(number + 1)} ${body.action} against ${body.uuid} on ${
          body.platform
        }.`,
      );
      await updateCache(app).catch(() => {
        return null;
      });
    } catch (error) {
      response.status(500);
      response.header("Content-Type", "application/json");
      response.send(JSON.stringify(error));
    }
  });

  server.post<{
    // eslint-disable-next-line @typescript-eslint/naming-convention -- Required convention for Fastify.
    Body: {
      case:   string;
      reason: string;
      token:  string;
    };
  }>("/revoke", async(request, response) => {
    try {
      const { body } = request;

      const token = await app.database.tokens.findUnique({
        where: {
          token: body.token,
        },
      });

      if (!token) {
        response.status(401);
        response.send("Invalid token.");
        return;
      }

      const number = Number.parseInt(body.case, 10);

      const sanction = await app.database.sanctions.findUnique({
        where: {
          number,
        },
      });

      if (!sanction) {
        response.status(404);
        response.send(`Sanction #${String(number)} not found.`);
        return;
      }

      await app.database.sanctions.update({
        data: {
          revokeDate:   new Date(),
          revokeReason: body.reason,
          revoked:      true,
          revokedBy:    token.username,
        },
        where: {
          number,
        },
      });
      response.status(200);
      response.send(
        `Revoked #${String(number)} ${sanction.action} against ${
          sanction.uuid
        } on ${sanction.platform}.`,
      );
      await updateCache(app).catch(() => {
        return null;
      });
    } catch (error) {
      response.status(500);
      response.header("Content-Type", "application/json");
      response.send(JSON.stringify(error, null, 2));
    }
  });

  await server.listen({ port: 12_443 });
};
